#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import argparse
import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "add"
    dt_add = sub.add_parser("add", help="Add a device type")
    dt_add.add_argument("name", help="Name of the device-type")
    dt_add.add_argument("--description", default=None,
                        help="Device-type description")
    dt_add.add_argument("--hide", dest="display", action="store_false",
                        default=True, help="Device is hidden in the UI")
    dt_add.add_argument("--owners-only", action="store_true", default=False,
                        help="Devices are only visible to owners")
    dt_health = dt_add.add_argument_group("health check")
    dt_health.add_argument("--health-frequency", default=24,
                           help="How often to run health checks.")
    dt_health.add_argument("--health-denominator", default="hours",
                           choices=["hours", "jobs"],
                           help="Initiate health checks by hours or by jobs.")

    # "aliases"
    dt_aliases = sub.add_parser("aliases", help="Manage aliases")
    aliases_sub = dt_aliases.add_subparsers(dest="sub_sub_sub_command",
                                            help="Sub commands")
    aliases_sub.required = True

    aliases_add = aliases_sub.add_parser("add", help="Add aliases")
    aliases_add.add_argument("name", help="Name of the device-type")
    aliases_add.add_argument("alias", help="Name of alias")

    aliases_delete = aliases_sub.add_parser("delete", help="Delete aliases")
    aliases_delete.add_argument("name", help="Name of the device-type")
    aliases_delete.add_argument("alias", help="Name of alias")

    aliases_list = aliases_sub.add_parser("list", help="List aliases for the device-type")
    aliases_list.add_argument("name", help="device-type")
    aliases_list.add_argument("--yaml", dest="output_format", default=None,
                              action="store_const", const="yaml",
                              help="Print as yaml")

    # "list"
    dt_list = sub.add_parser("list", help="List available device-types")
    dt_list.add_argument("--all", "-a", dest="show_all",
                         default=False, action="store_true",
                         help="Show all device types in the database, "
                              "including non-installed ones")
    dt_list.add_argument("--yaml", dest="output_format", default=None,
                         action="store_const", const="yaml",
                         help="Print as yaml")

    # "show"
    dt_show = sub.add_parser("show", help="Show device-type details")
    dt_show.add_argument("name", help="Name of the device-type")
    dt_show.add_argument("--yaml", dest="output_format",
                         action="store_const", const="yaml",
                         default=None, help="Print as yaml")

    # "template"
    devices_template = sub.add_parser("template", help="Device-type template")
    dt_sub = devices_template.add_subparsers(dest="sub_sub_sub_command",
                                             help="Sub commands")
    dt_sub.required = True
    dt_get = dt_sub.add_parser("get",
                               help="Get the device-type template")
    dt_get.add_argument("name", help="Name of the device-type")

    dt_set = dt_sub.add_parser("set",
                               help="Set the device-type template")
    dt_set.add_argument("name", help="Name of the device-type")
    dt_set.add_argument("template", type=argparse.FileType('r'),
                        help="Device-type template")

    # "update"
    dt_update = sub.add_parser("update", help="Update device-type")
    dt_update.add_argument("name", help="Name of the device-type")
    dt_update.add_argument("--description", default=None,
                           help="Device-type description")

    visibility = dt_update.add_mutually_exclusive_group()
    visibility.add_argument("--hide", dest="display", action="store_false",
                            default=None, help="Device-type is hidden in the UI")
    visibility.add_argument("--show", dest="display", action="store_true",
                            help="Device-type is visible in the UI")

    owner = dt_update.add_mutually_exclusive_group()
    owner.add_argument("--owners-only", action="store_true", dest="owners_only",
                       default=None,
                       help="Devices are only visible to owners")
    owner.add_argument("--public", action="store_false", dest="owners_only",
                       help="Devices are visible to all users")

    dt_health = dt_update.add_argument_group("health check")
    dt_health.add_argument("--health-frequency", default=None,
                           help="How often to run health checks.")
    dt_health.add_argument("--health-denominator", default=None,
                           choices=["hours", "jobs"],
                           help="Initiate health checks by hours or by jobs.")

    health = dt_health.add_mutually_exclusive_group()
    health.add_argument("--health-disabled", default=None,
                        action="store_true",
                        help="Disable health checks")
    health.add_argument("--health-active", dest="health_disabled",
                        action="store_false",
                        help="Activate health checks")


def help():
    return "Manage device-types"


def handle_add(proxy, options):
    proxy.scheduler.device_types.add(options.name, options.description,
                                     options.display, options.owners_only,
                                     options.health_frequency,
                                     options.health_denominator)


def handle_aliases(proxy, options):
    if options.sub_sub_sub_command == "add":
        proxy.scheduler.device_types.aliases.add(options.name, options.alias)
    elif options.sub_sub_sub_command == "list":
        aliases = proxy.scheduler.device_types.aliases.list(options.name)
        if options.output_format == "yaml":
            print(yaml.dump(aliases).rstrip("\n"))
        else:
            print("Aliases:")
            for alias in aliases:
                print("* %s" % alias)
    elif options.sub_sub_sub_command == "delete":
        proxy.scheduler.device_types.aliases.delete(options.name,
                                                    options.alias)


def handle_list(proxy, options):
    device_types = proxy.scheduler.device_types.list(options.show_all)

    if options.output_format == "yaml":
        print(yaml.dump(device_types).rstrip("\n"))
    else:
        print("Device-Types:")
        for dt in device_types:
            print("* %s (%s)" % (dt["name"], dt["devices"]))


def handle_show(proxy, options):
    dt = proxy.scheduler.device_types.show(options.name)

    if options.output_format == "yaml":
        print(yaml.dump(dt).rstrip("\n"))
    else:
        print("name            : %s" % dt["name"])
        print("description     : %s" % dt["description"])
        print("display         : %s" % dt["display"])
        print("owners only     : %s" % dt["owners_only"])
        print("health disabled : %s" % dt["health_disabled"])
        print("aliases         : %s" % dt["aliases"])
        print("devices         : %s" % dt["devices"])


def handle_template(proxy, options):
    if options.sub_sub_sub_command == "get":
        template = proxy.scheduler.device_types.get_template(options.name)
        print(str(template).rstrip("\n"))
    else:
        template = options.template.read()
        proxy.scheduler.device_types.set_template(options.name, template)


def handle_update(proxy, options):
    proxy.scheduler.device_types.update(options.name, options.description,
                                        options.display, options.owners_only,
                                        options.health_frequency,
                                        options.health_denominator,
                                        options.health_disabled)


def handle(proxy, options, _):
    handlers = {
        "add": handle_add,
        "aliases": handle_aliases,
        "list": handle_list,
        "show": handle_show,
        "template": handle_template,
        "update": handle_update
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "add"
    aliases_add = sub.add_parser("add", help="Add an alias")
    aliases_add.add_argument("alias", help="Alias name")

    # "delete"
    aliases_del = sub.add_parser("delete", help="Delete an alias")
    aliases_del.add_argument("alias", help="Alias name")

    # "list"
    aliases_list = sub.add_parser("list", help="List available aliases")
    aliases_list.add_argument("--yaml", dest="output_format", default=None,
                              action="store_const", const="yaml",
                              help="Print as yaml")

    # "show"
    aliases_show = sub.add_parser("show", help="Show alias details")
    aliases_show.add_argument("alias", help="Alias")
    aliases_show.add_argument("--yaml", dest="output_format",
                              action="store_const", const="yaml",
                              default=None, help="Print as yaml")


def help():
    return "Manage aliases"


def handle_add(proxy, options):
    proxy.scheduler.aliases.add(options.alias)


def handle_delete(proxy, options):
    proxy.scheduler.aliases.delete(options.alias)


def handle_list(proxy, options):
    aliases = proxy.scheduler.aliases.list()
    if options.output_format == "yaml":
        print(yaml.dump(aliases).rstrip("\n"))
    else:
        print("Aliases:")
        for alias in aliases:
            print("* %s" % alias)


def handle_show(proxy, options):
    alias = proxy.scheduler.aliases.show(options.alias)
    if options.output_format == "yaml":
        print(yaml.dump(alias).rstrip("\n"))
    else:
        print("name        : %s" % alias["name"])
        print("device-types:")
        for dt in alias["device_types"]:
            print("* %s" % dt)


def handle(proxy, options, _):
    handlers = {
        "add": handle_add,
        "delete": handle_delete,
        "list": handle_list,
        "show": handle_show
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

from contextlib import suppress
import os
import shutil
import xmlrpc.client
import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "export"
    sys_export = sub.add_parser("export", help="Export server configuration")
    sys_export.add_argument("name", help="Name of the export")

    # "import"
    sub.add_parser("import", help="Import server configuration")

    # "methods"
    sys_methods = sub.add_parser("methods", help="List methods")
    sys_sub = sys_methods.add_subparsers(dest="sub_sub_sub_command",
                                         help="Sub commands")
    sys_sub.required = True
    sys_sub.add_parser("list", help="List available methods")

    sys_help = sys_sub.add_parser("help", help="Method help")
    sys_help.add_argument("method", help="Method name")

    sys_signature = sys_sub.add_parser("signature", help="Method signature")
    sys_signature.add_argument("method", help="Method name")

    # "version"
    sub.add_parser("version", help="Return the server version")

    # "whoami"
    sub.add_parser("whoami", help="Return the current username")


def help():
    return "System information"


def handle_export(proxy, options):
    print("Export to %s" % options.name)
    with suppress(FileNotFoundError):
        shutil.rmtree(options.name)
    os.mkdir(options.name)
    os.chdir(options.name)

    print("Listing aliases")
    aliases = []
    for alias in proxy.scheduler.aliases.list():
        print("* %s" % alias)
        aliases.append(proxy.scheduler.aliases.show(alias))

    print("Listing tags")
    tags = []
    for tag in proxy.scheduler.tags.list():
        print("* %s" % tag["name"])
        tags.append(tag)

    print("Listing workers")
    workers = []
    for worker in proxy.scheduler.workers.list():
        print("* %s" % worker)
        w = proxy.scheduler.workers.show(worker)
        workers.append({"hostname": w["hostname"],
                        "description": w["description"],
                        "master": w["master"],
                        "hidden": w["hidden"]})

    print("Listing device-types")
    os.mkdir("device-types")
    device_types = []
    for device_type in proxy.scheduler.device_types.list():
        print("* %s" % device_type["name"])
        dt = proxy.scheduler.device_types.show(device_type["name"])
        device_types.append({"name": dt["name"],
                             "description": dt["description"],
                             "display": dt["display"],
                             "health_disabled": dt["health_disabled"],
                             "owners_only": dt["owners_only"],
                             "aliases": dt["aliases"]})

        try:
            dt_template = proxy.scheduler.device_types.get_template(dt["name"])
        except xmlrpc.client.Fault as exc:
            if exc.faultCode == 404:
                print("  => No template found")
                continue
            raise
        with open(os.path.join("device-types", dt["name"] + ".jinja2"), "w") as f_out:
            f_out.write(str(dt_template))

    print("Listing devices")
    os.mkdir("devices")
    devices = []
    for device in proxy.scheduler.devices.list():
        print("* %s" % device["hostname"])
        d = proxy.scheduler.devices.show(device["hostname"])
        devices.append({"hostname": d["hostname"],
                        "description": d["description"],
                        "device_type": d["device_type"],
                        "pipeline": d["pipeline"],
                        "worker": d["worker"],
                        "status": d["status"],
                        "public": d["public"],
                        "user": d["user"],
                        "group": d["group"],
                        "health": d["health"],
                        "tags": d["tags"]})

        try:
            device_dict = proxy.scheduler.devices.get_dictionary(device["hostname"])
        except xmlrpc.client.Fault as exc:
            if exc.faultCode == 404:
                print("  => No device dict found")
                continue
            raise
        with open(os.path.join("devices", device["hostname"] + ".jinja2"), "w") as f_out:
            f_out.write(str(device_dict))

    export = {"aliases": aliases,
              "devices": devices,
              "device-types": device_types,
              "tags": tags,
              "workers": workers}

    # Dump the configuration
    with open("instance.yaml", "w") as f_out:
        f_out.write(yaml.dump(export).rstrip("\n"))


def handle_methods(proxy, options):
    if options.sub_sub_sub_command == "help":
        print(proxy.system.methodHelp(options.method))
    elif options.sub_sub_sub_command == "signature":
        print(proxy.system.methodSignature(options.method))
    else:
        # Fallback to "list"
        methods = proxy.system.listMethods()
        for method in methods:
            print(method)


def handle_version(proxy, _):
    print(proxy.system.version())


def handle_whoami(proxy, _):
    username = proxy.system.whoami()
    if username is None:
        print("<AnonymousUser>")
    else:
        print(username)


def handle(proxy, options, _):
    handlers = {
        "export": handle_export,
        "methods": handle_methods,
        "version": handle_version,
        "whoami": handle_whoami
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import argparse
import yaml

from . import jobs


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "add"
    devices_add = sub.add_parser("add", help="Add a device")
    devices_add.add_argument("hostname", help="Hostname of the device")
    devices_add.add_argument("--type", required=True, help="Device-type")
    devices_add.add_argument("--worker", required=True, help="Worker hostname")
    devices_add.add_argument("--description", default=None,
                             help="Device description")

    owner = devices_add.add_mutually_exclusive_group()
    owner.add_argument("--user", default=None, help="Device owner")
    owner.add_argument("--group", default=None, help="Device owner")

    devices_add.add_argument("--status", default=None,
                             choices=["OFFLINE", "IDLE", "RUNNING",
                                      "OFFLINING", "RETIRED", "RESERVED"],
                             help="Device status")
    devices_add.add_argument("--health", default=None,
                             choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
                             help="Device health status")

    devices_add.add_argument("--private", action="store_true", default=False,
                             help="Private device, public by default")

    # "dict"
    devices_dict = sub.add_parser("dict", help="Device dictionary")
    dict_sub = devices_dict.add_subparsers(dest="sub_sub_sub_command",
                                               help="Sub commands")
    dict_sub.required = True
    dict_get = dict_sub.add_parser("get",
                                   help="Get the device dictionary")
    dict_get.add_argument("hostname", help="Hostname of the device")
    dict_get.add_argument("--render", action="store_true", default=False,
                          help="Render the dictionary to the configuration.")

    dict_set = dict_sub.add_parser("set",
                                   help="Set the device dictionary")
    dict_set.add_argument("hostname", help="Hostname of the device")
    dict_set.add_argument("config", type=argparse.FileType('r'),
                          help="Configuration file")

    # "force-health"
    devices_fh = sub.add_parser("force-health", help="Force a health check for the device")
    devices_fh.add_argument("hostname", help="Device hostname")
    devices_fh.add_argument("--follow", action="store_true", default=False,
                            help="Wait for the job and print the logs")

    # "list"
    devices_list = sub.add_parser("list", help="List available devices")
    devices_list.add_argument("--all", action="store_true", default=False,
                              help="List every devices, inluding retired")
    devices_list.add_argument("--yaml", dest="output_format", default=None,
                              action="store_const", const="yaml",
                              help="Print as yaml")

    # "show"
    devices_show = sub.add_parser("show", help="Show device details")
    devices_show.add_argument("hostname", help="Device hostname")
    devices_show.add_argument("--yaml", dest="output_format",
                              action="store_const", const="yaml",
                              default=None, help="Print as yaml")

    # "tags"
    devices_tags = sub.add_parser("tags", help="Manage tags")
    tags_sub = devices_tags.add_subparsers(dest="sub_sub_sub_command",
                                           help="Sub commands")
    tags_sub.required = True
    tags_add = tags_sub.add_parser("add", help="Add a tag")
    tags_add.add_argument("hostname", help="Hostname of the device")
    tags_add.add_argument("tag", help="Name of the tag")

    tags_list = tags_sub.add_parser("list", help="List tags for the device")
    tags_list.add_argument("hostname", help="Hostname of the device")
    tags_list.add_argument("--yaml", dest="output_format",
                           action="store_const", const="yaml",
                           help="Print as yaml")

    tags_del = tags_sub.add_parser("delete", help="Remove a tag")
    tags_del.add_argument("hostname", help="Hostname of the device")
    tags_del.add_argument("tag", help="Name of the tag")

    # "update"
    devices_update = sub.add_parser("update", help="Update device properties")
    devices_update.add_argument("hostname",
                                help="Hostname of the device")
    devices_update.add_argument("--worker", default=None,
                                help="Update the worker")
    devices_update.add_argument("--description", default=None,
                                help="Update the description")

    owner = devices_update.add_mutually_exclusive_group()
    owner.add_argument("--user", default=None, help="Device owner")
    owner.add_argument("--group", default=None, help="Device owner")

    devices_update.add_argument("--status", default=None,
                                choices=["OFFLINE", "IDLE", "RUNNING",
                                         "OFFLINING", "RETIRED", "RESERVED"],
                                help="Update the device status")
    devices_update.add_argument("--health", default=None,
                                choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
                                help="Update the device health status")

    display = devices_update.add_mutually_exclusive_group()
    display.add_argument("--public", default=None, action="store_true",
                         help="Make the device public")
    display.add_argument("--private", dest="public", action="store_false",
                         help="Make the device private")


def help():
    return "Manage devices"


def handle_add(proxy, options):
    proxy.scheduler.devices.add(options.hostname, options.type,
                                options.worker, options.user,
                                options.group, not options.private,
                                options.status, options.health,
                                options.description)


def handle_dict(proxy, options):
    if options.sub_sub_sub_command == "get":
        config = proxy.scheduler.devices.get_dictionary(options.hostname,
                                                        options.render)
        print(str(config).rstrip("\n"))
    else:
        config = options.config.read()
        ret = proxy.scheduler.devices.set_dictionary(options.hostname,
                                                     config)
        # TODO: return the right value
        if not ret:
            print("Unable to set the configuration")


def handle_fh(proxy, options):
    job_id = proxy.scheduler.devices.force_health_check(options.hostname)
    if options.follow:
        # Adapt to "jobs logs" options
        options.job_id = job_id
        options.no_follow = False
        options.polling = 5
        options.raw = False
        jobs.handle_logs(proxy, options)


def handle_list(proxy, options):
    devices = proxy.scheduler.devices.list(options.all)

    if options.output_format == "yaml":
        print(yaml.dump(devices).rstrip("\n"))
    else:
        print("Devices:")
        for device in devices:
            print("* %s (%s): %s" % (device["hostname"], device["type"], device["status"]))


def handle_show(proxy, options):
    device = proxy.scheduler.devices.show(options.hostname)

    if options.output_format == "yaml":
        print(yaml.dump(device).rstrip("\n"))
    else:
        print("name        : %s" % device["hostname"])
        print("device-type : %s" % device["device_type"])
        print("status      : %s" % device["status"])
        print("user        : %s" % device["user"])
        print("group       : %s" % device["group"])
        print("health      : %s" % device["health"])
        print("health job  : %s" % device["health_job"])
        print("description : %s" % device["description"])
        print("public      : %s" % device["public"])
        print("pipeline    : %s" % device["pipeline"])
        print("device-dict : %s" % device["has_device_dict"])
        print("worker      : %s" % device["worker"])
        print("current job : %s" % device["current_job"])
        print("tags        : %s" % device["tags"])


def handle_tags(proxy, options):
    if options.sub_sub_sub_command == "add":
        proxy.scheduler.devices.tags.add(options.hostname, options.tag)
    elif options.sub_sub_sub_command == "delete":
        proxy.scheduler.devices.tags.delete(options.hostname, options.tag)
    else:
        tags = proxy.scheduler.devices.tags.list(options.hostname)
        if options.output_format == "yaml":
            print(yaml.dump(tags).rstrip("\n"))
        else:
            print("Tags:")
            for tag in tags:
                print("* %s" % tag)


def handle_update(proxy, options):
    proxy.scheduler.devices.update(options.hostname, options.worker,
                                   options.user, options.group,
                                   options.public, options.status,
                                   options.health, options.description)


def handle(proxy, options, _):
    handlers = {
        "add": handle_add,
        "dict": handle_dict,
        "force-health": handle_fh,
        "list": handle_list,
        "show": handle_show,
        "tags": handle_tags,
        "update": handle_update
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import json


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "listen"
    sub.add_parser("listen", help="Listen to events")

    # "wait"
    sub.add_parser("wait", help="Wait for a specifi event")
    # TODO: add the right arguments


def help():
    return "Listen to events"


def handle_listen(proxy, options, config):
    import zmq
    from zmq.utils.strtypes import b, u
    # Try to find the socket url
    if config is None or config.get("events", {}).get("uri") is None:
        url = proxy.scheduler.get_publisher_event_socket()
    else:
        url = config["events"]["uri"]

    if url is None:
        print("Unable to find the socket url")
        return 1


    context = zmq.Context()
    sock = context.socket(zmq.SUB)
    sock.setsockopt(zmq.SUBSCRIBE, b"")
    # Set the sock proxy (if needed)
    socks = config.get("events", {}).get("socks_proxy")
    if socks is not None:
        print("Listening to %s (socks %s)" % (url, socks))
        sock.setsockopt(zmq.SOCKS_PROXY, b(socks))
    else:
        print("Listening to %s" % url)

    try:
        sock.connect(url)
    except zmq.error.ZMQError as exc:
        print("Unable to connect: %s" % exc)
        return 1

    while True:
        msg = sock.recv_multipart()
        try:
            (topic, uuid, dt, username, data) = (u(m) for m in msg)
        except ValueError:
            print("Invalid message: %s" % msg)
            continue

        # If unknown, print the full data
        msg = data
        data = json.loads(data)
        # Print according to the topic
        topic_end = topic.split(".")[-1]
        if topic_end == "device":
            msg = "[%s] <%s> %s" % (data["device"], data["device_type"], data["status"])
            if "job" in data:
                msg += " for %s" % data["job"]
        elif topic_end == "testjob":
            msg = "[%s] <%s> %s (%s)" % (data["job"], data.get("device", "??"),
                                         data["status"], data["description"])
        print("\033[1;30m%s\033[0m \033[1;37m%s\033[0m \033[32m%s\033[0m - %s" % (dt, topic, username, msg))


def handle_wait(proxy, options, config):
    pass


def handle(proxy, options, config):
    handlers = {
        "listen": handle_listen,
        "wait": handle_wait
    }
    return handlers[options.sub_sub_command](proxy, options, config)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "add"
    tags_add = sub.add_parser("add", help="Add a tag")
    tags_add.add_argument("tag", help="Tag name")
    tags_add.add_argument("--description", default=None,
                          help="Tag description")

    # "delete"
    tags_delete = sub.add_parser("delete", help="Delete a tag")
    tags_delete.add_argument("tag", help="Tag name")

    # "list"
    tags_list = sub.add_parser("list", help="List tags")
    tags_list.add_argument("--yaml", dest="output_format",
                           action="store_const", const="yaml",
                           help="Print as yaml")

    # "show"
    tags_show = sub.add_parser("show", help="Show tag details")
    tags_show.add_argument("tag", help="Tag name")
    tags_show.add_argument("--yaml", dest="output_format",
                           action="store_const", const="yaml",
                           help="Print as yaml")


def help():
    return "Manage tags"


def handle_add(proxy, options):
    proxy.scheduler.tags.add(options.tag, options.description)


def handle_delete(proxy, options):
    proxy.scheduler.tags.delete(options.tag)


def handle_list(proxy, options):
    tags = proxy.scheduler.tags.list()
    if options.output_format == "yaml":
        print(yaml.dump(tags).rstrip("\n"))
    else:
        print("Tags:")
        for tag in tags:
            if tag["description"]:
                print("* %s (%s)" % (tag["name"], tag["description"]))
            else:
                print("* %s" % tag["name"])


def handle_show(proxy, options):
    tag = proxy.scheduler.tags.show(options.tag)
    if options.output_format == "yaml":
        print(yaml.dump(tag).rstrip("\n"))
    else:
        print("name       : %s" % tag["name"])
        print("description: %s" % tag["description"])
        print("devices    :")
        for device in tag["devices"]:
            print("* %s" % device)


def handle(proxy, options, _):
    handlers = {
        "add":  handle_add,
        "delete": handle_delete,
        "list": handle_list,
        "show": handle_show
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import argparse
import os
import requests
import socket
import sys
from urllib.parse import urlparse
import xmlrpc.client
import yaml

from lavacli.commands import (
    aliases,
    devices,
    device_types,
    events,
    jobs,
    results,
    system,
    tags,
    workers
)


# Version string
__version__ = "0.1"


class RequestsTransport(xmlrpc.client.Transport):

    def __init__(self, scheme, proxy=None, timeout=20.0, verify_ssl_cert=True):
        super().__init__()
        self.scheme = scheme
        # Set the user agent
        self.user_agent = "lavacli v%s" % __version__
        if proxy is None:
            self.proxies = {}
        else:
            self.proxies = {scheme: proxy}
        self.timeout = timeout
        self.verify_ssl_cert = verify_ssl_cert
        if not verify_ssl_cert:
            from requests.packages.urllib3.exceptions import InsecureRequestWarning
            requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

    def request(self, host, handler, data, verbose):
        headers = {"User-Agent": self.user_agent,
                   "Content-Type": "text/xml",
                   "Accept-Encoding": "gzip"}
        url = "%s://%s%s" % (self.scheme, host, handler)
        try:
            response = None
            response = requests.post(url, data=data, headers=headers,
                                     timeout=self.timeout,
                                     verify=self.verify_ssl_cert,
                                     proxies=self.proxies)
            response.raise_for_status()
            return self.parse_response(response)
        except requests.RequestException as e:
            if response is None:
                raise xmlrpc.client.ProtocolError(url, 500, str(e), "")
            else:
                raise xmlrpc.client.ProtocolError(url, response.status_code,
                                                  str(e), response.headers)

    def parse_response(self, resp):
        """
        Parse the xmlrpc response.
        """
        p, u = self.getparser()
        p.feed(resp.text)
        p.close()
        return u.close()


def load_config(identity):
    # Build the path to the configuration file
    config_dir = os.environ.get("XDG_CONFIG_HOME") or "~/.config"
    config_filename = os.path.expanduser(os.path.join(config_dir,
                                                      "lavacli.yaml"))

    # load configuration from file
    with open(config_filename, "r") as f_conf:
        config = yaml.load(f_conf.read())

    return config.get(identity)


def parser(commands):
    parser_obj = argparse.ArgumentParser()
    url = parser_obj.add_mutually_exclusive_group(required=True)
    url.add_argument("--uri", type=str, default=None,
                     help="URI of the lava-server RPC endpoint")
    url.add_argument("--identity", "-i", type=str, default=None,
                     help="Server identity")

    root = parser_obj.add_subparsers(dest="sub_command", help="Sub commands")

    keys = list(commands.keys())
    keys.sort()
    for name in keys:
        cls = commands[name]
        cls.configure_parser(root.add_parser(name, help=cls.help()))

    return parser_obj


def main():
    commands = {"aliases": aliases,
                "devices": devices,
                "device-types": device_types,
                "events": events,
                "jobs": jobs,
                "results": results,
                "system": system,
                "tags": tags,
                "workers": workers}

    # Parse the command line
    parser_obj = parser(commands)
    options = parser_obj.parse_args()

    if options.uri is None:
        config = load_config(options.identity)
        if config is None:
            print("Unknown identity '%s'" % options.identity)
            return 1
        options.uri = config["uri"]
    else:
        config = None

    # Check that a sub_command was given
    if options.sub_command is None:
        parser_obj.print_help()
        return 1

    # Create the Transport object
    parsed_uri = urlparse(options.uri)
    if options.identity:
        transport = RequestsTransport(parsed_uri.scheme,
                                      config.get("proxy"),
                                      config.get("timeout", 20.0),
                                      config.get("verify_ssl_cert", True))
    else:
        transport = RequestsTransport(parsed_uri.scheme)

    # Connect to the RPC endpoint
    try:
        # allow_none is True because the server does support it
        proxy = xmlrpc.client.ServerProxy(options.uri, allow_none=True,
                                          transport=transport)
        # FIXME: return the right value
        return commands[options.sub_command].handle(proxy, options, config)
    except xmlrpc.client.Error as exc:
        if "sub_sub_command" in options:
            print("Unable to call '%s.%s': %s" % (options.sub_command,
                                                  options.sub_sub_command,
                                                  str(exc)))
        else:
            print("Unable to call '%s': %s" % (options.sub_command,
                                               str(exc)))
    except (ConnectionError, socket.gaierror) as exc:
        print("Unable to connect to '%s': %s" % (options.uri, str(exc)))

    return 1



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import sys
import yaml


def configure_parser(parser):
    parser.add_argument("job_id", help="Job id")
    parser.add_argument("test_suite", nargs="?", default=None, help="Test suite")
    parser.add_argument("test_case", nargs="?", default=None, help="Test case")
    parser.add_argument("--yaml", dest="output_format", default=None,
                        action="store_const", const="yaml",
                        help="Print as yaml")


def help():
    return "Manage results"


def handle(proxy, options, _):
    if options.test_case is not None:
        data = proxy.results.get_testcase_results_yaml(options.job_id,
                                                       options.test_suite,
                                                       options.test_case)
    elif options.test_suite is not None:
        data = proxy.results.get_testsuite_results_yaml(options.job_id,
                                                        options.test_suite)
    else:
        data = proxy.results.get_testjob_results_yaml(options.job_id)

    results = yaml.load(data)

    if options.output_format == "yaml":
        print(yaml.dump(results).rstrip("\n"))
    else:
        # Only one to print
        if options.test_case is not None:
            res = results[0]
            if not sys.stdout.isatty():
                print("%s" % res["result"])
            elif res["result"] == "pass":
                print("\033[1;32mpass\033[0m")
            elif res["result"] == "fail":
                print("\033[1;31mfail\033[0m")
            else:
                print("%s" % res["result"])
        # A list to print
        else:
            print("Results:")
            for res in results:
                if not sys.stdout.isatty():
                    print("* %s.%s [%s]" % (res["suite"], res["name"], res["result"]))
                elif res["result"] == "pass":
                    print("* %s.%s [\033[1;32mpass\033[0m]" % (res["suite"], res["name"]))
                elif res["result"] == "fail":
                    print("* %s.%s [\033[1;31mfail\033[0m]" % (res["suite"], res["name"]))
                else:
                    print("* %s.%s [%s]" % (res["suite"], res["name"], res["result"]))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import argparse
import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "add"
    workers_add = sub.add_parser("add", help="Add a worker")
    workers_add.add_argument("hostname", type=str,
                             help="Hostname of the worker")
    workers_add.add_argument("--description", type=str, default=None,
                             help="Worker description")
    workers_add.add_argument("--disabled", action="store_true",
                             default=False, help="Create a disabled worker")


    # "config"
    workers_config = sub.add_parser("config", help="Worker configuration")
    config_sub = workers_config.add_subparsers(dest="sub_sub_sub_command",
                                               help="Sub commands")
    config_sub.required = True
    config_get = config_sub.add_parser("get",
                                       help="Get the worker configuration")
    config_get.add_argument("hostname", type=str,
                            help="Hostname of the worker")

    config_set = config_sub.add_parser("set",
                                       help="Set the worker configuration")
    config_set.add_argument("hostname", type=str,
                            help="Hostname of the worker")
    config_set.add_argument("config", type=argparse.FileType('r'),
                            help="Configuration file")

    # "list"
    workers_list = sub.add_parser("list", help="List workers")
    workers_list.add_argument("--yaml", dest="output_format",
                              action="store_const", const="yaml",
                              default=None, help="Print as yaml")

    # "update"
    update_parser = sub.add_parser("update", help="Update worker properties")
    update_parser.add_argument("hostname", type=str,
                               help="Hostname of the worker")
    update_parser.add_argument("--description", type=str, default=None,
                               help="Worker description")
    display = update_parser.add_mutually_exclusive_group()
    display.add_argument("--disable", action="store_false",
                         default=None, dest="display",
                         help="Disable the worker")
    display.add_argument("--enable", action="store_true",
                         default=None, dest="display",
                         help="Enable the worker")

    # "show"
    workers_show = sub.add_parser("show", help="Details of a worker")
    workers_show.add_argument("hostname", help="Hostname of the worker")
    workers_show.add_argument("--yaml", dest="output_format",
                              action="store_const", const="yaml",
                              default=None, help="Print as yaml")


def help():
    return "Manage workers"


def handle_add(proxy, options):
    proxy.scheduler.workers.add(options.hostname,
                                options.description,
                                options.disabled)


def handle_config(proxy, options):
    if options.sub_sub_sub_command == "get":
        config = proxy.scheduler.workers.get_config(options.hostname)
        print(str(config).rstrip("\n"))
    else:
        config = options.config.read()
        ret = proxy.scheduler.workers.set_config(options.hostname,
                                                 config)
        if not ret:
            print("Unable to store worker configuration")
            return 1


def handle_list(proxy, options):
    workers = proxy.scheduler.workers.list()
    if options.output_format == "yaml":
        print(yaml.dump(workers).rstrip("\n"))
    else:
        print("Workers:")
        for worker in workers:
            print("* %s" % worker)


def handle_show(proxy, options):
    worker = proxy.scheduler.workers.show(options.hostname)
    if options.output_format == "yaml":
        print(yaml.dump(worker).rstrip("\n"))
    else:
        print("hostname    : %s" % worker["hostname"])
        print("description : %s" % worker["description"])
        print("master      : %s" % worker["master"])
        print("hidden      : %s" % worker["hidden"])
        print("devices     : %s" % worker["devices"])


def handle_update(proxy, options):
    proxy.scheduler.workers.update(options.hostname,
                                   options.description,
                                   not options.display)


def handle(proxy, options, _):
    handlers = {
        "add": handle_add,
        "config": handle_config,
        "list": handle_list,
        "show": handle_show,
        "update": handle_update
    }
    return handlers[options.sub_sub_command](proxy, options)



# TODO: use zmq events!
#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

import argparse
import datetime
import sys
import time
import yaml


def configure_parser(parser):
    sub = parser.add_subparsers(dest="sub_sub_command", help="Sub commands")
    sub.required = True

    # "cancel"
    jobs_cancel = sub.add_parser("cancel", help="Cancel job")
    jobs_cancel.add_argument("job_id", help="Job id")

    jobs_definition = sub.add_parser("definition", help="Job definition")
    jobs_definition.add_argument("job_id", help="Job id")

    # "list"
    jobs_list = sub.add_parser("list", help="List jobs")
    jobs_list.add_argument("--limit", type=int, default=10,
                           help="Limit to N jobs")
    jobs_list.add_argument("--yaml", dest="output_format", default=None,
                           action="store_const", const="yaml",
                           help="Print as yaml")

    # "logs"
    jobs_logs = sub.add_parser("logs", help="Get logs")
    jobs_logs.add_argument("job_id", help="Job id")
    jobs_logs.add_argument("--no-follow", default=False, action="store_true",
                           help="Do not keep polling until the end of the job")
    jobs_logs.add_argument("--polling", default=5, type=int,
                           help="Polling in seconds, 5s by default")
    jobs_logs.add_argument("--raw", default=False, action="store_true",
                           help="Print raw logs")

    # "run"
    jobs_run = sub.add_parser("run", help="Run the job")
    jobs_run.add_argument("definition", type=argparse.FileType('r'),
                          help="Job definition")
    jobs_run.add_argument("--no-follow", default=False, action="store_true",
                          help="Do not keep polling until the end of the job")
    jobs_run.add_argument("--polling", default=5, type=int,
                          help="Polling in seconds, 5s by default")
    jobs_run.add_argument("--raw", default=False, action="store_true",
                          help="Print raw logs")

    # "show"
    jobs_show = sub.add_parser("show", help="Job details")
    jobs_show.add_argument("job_id", help="Job id")
    jobs_show.add_argument("--yaml", dest="output_format",
                           action="store_const", const="yaml",
                           default=None, help="Print as yaml")

    # "resubmit"
    jobs_resubmit = sub.add_parser("resubmit", help="Resubmit job")
    jobs_resubmit.add_argument("job_id", help="Job id")
    jobs_resubmit.add_argument("--follow", default=True, dest="no_follow",
                               action="store_false",
                               help="Resubmit and poll for the logs")
    jobs_resubmit.add_argument("--polling", default=5, type=int,
                               help="Polling in seconds, 5s by default")
    jobs_resubmit.add_argument("--raw", default=False, action="store_true",
                               help="Print raw logs")

    # "submit"
    jobs_submit = sub.add_parser("submit", help="Submit job")
    jobs_submit.add_argument("definition", type=argparse.FileType('r'),
                             help="Job definition")


def help():
    return "Manage jobs"


def handle_cancel(proxy, options):
    proxy.scheduler.jobs.cancel(options.job_id)


def handle_definition(proxy, options):
    print(proxy.scheduler.jobs.definition(options.job_id))


def handle_list(proxy, options):
    jobs = proxy.scheduler.jobs.list(options.limit)
    if options.output_format == "yaml":
        print(yaml.dump(jobs).rstrip("\n"))
    else:
        print("Jobs:")
        for job in jobs:
            print("* %s: %s [%s] (%s)" % (job["id"], job["status"],
                                          job["submitter"],
                                          job["description"]))


if sys.stdout.isatty():
    COLORS = {"exception": "\033[1;31m]",
              "error": "\033[1;31m",
              "warning": "\033[1;33m",
              "info": "\033[1;37m",
              "debug": "",
              "target": "\033[32m",
              "input": "\033[0;35m",
              "feedback": "\033[0;33m",
              "results": "\033[1;34m",
              "dt": "\033[1;30m",
              "end": "\033[0m"}
else:
    COLORS = {"exception": "",
              "error": "",
              "warning": "",
              "info": "",
              "debug": "",
              "target": "",
              "input": "",
              "feedback": "",
              "results": "",
              "dt": "",
              "end": ""}


def print_logs(logs, raw):
    if raw:
        for line in logs:
            print(line)
    else:
        for line in logs:
            timestamp = line["dt"].split(".")[0]
            level = line["lvl"]
            if isinstance(line["msg"], dict) and \
               "sending" in line["msg"].keys():
                level = "input"
                msg = str(line["msg"]["sending"])
            else:
                msg = str(line["msg"])
            msg = msg.rstrip("\n")

            print(COLORS["dt"] + timestamp + COLORS["end"] + " " +
                  COLORS[level] + msg + COLORS["end"])


def handle_logs(proxy, options):
    # Loop
    lines = 0
    while True:
        (finished, data) = proxy.scheduler.jobs.logs(options.job_id, lines)
        logs = yaml.load(str(data), Loader=yaml.CLoader)
        if logs:
            print_logs(logs, options.raw)
            lines += len(logs)
        # Loop only if the job is not finished
        if finished or options.no_follow:
            return

        # Wait some time
        time.sleep(options.polling)


def handle_resubmit(proxy, options):
    job_id = proxy.scheduler.jobs.resubmit(options.job_id)

    if options.no_follow:
        if isinstance(job_id, list):
            for job in job_id:
                print(job)
        else:
            print(job_id)

    else:
        print_logs([{"dt": datetime.datetime.utcnow().isoformat(),
                     "lvl": "debug",
                     "msg": "Job %s submitted" % job_id}],
                   options.raw)

        # Add the job_id to options for handle_logs
        # For multinode, print something and loop on all jobs
        if isinstance(job_id, list):
            for job in job_id:
                print_logs([{"dt": datetime.datetime.utcnow().isoformat(),
                             "lvl": "debug",
                             "msg": "Seeing %s logs" % job}],
                           options.raw)
                options.job_id = job
                handle_logs(proxy, options)
        else:
            options.job_id = job_id
            handle_logs(proxy, options)


def handle_run(proxy, options):
    job_id = proxy.scheduler.jobs.submit(options.definition.read())
    print_logs([{"dt": datetime.datetime.utcnow().isoformat(),
                 "lvl": "debug",
                 "msg": "Job %s submitted" % job_id}],
               options.raw)

    # Add the job_id to options for handle_logs
    # For multinode, print something and loop on all jobs
    if isinstance(job_id, list):
        for job in job_id:
            print_logs([{"dt": datetime.datetime.utcnow().isoformat(),
                         "lvl": "debug",
                         "msg": "Seeing %s logs" % job}],
                       options.raw)
            options.job_id = job
            handle_logs(proxy, options)
    else:
        options.job_id = job_id
        handle_logs(proxy, options)


def handle_show(proxy, options):
    job = proxy.scheduler.jobs.show(options.job_id)

    if options.output_format == "yaml":
        job["submit_time"] = job["submit_time"].value
        job["start_time"] = job["start_time"].value
        job["end_time"] = job["end_time"].value
        print(yaml.dump(job).rstrip("\n"))
    else:
        print("id          : %s" % job["id"])
        print("description : %s" % job["description"])
        print("submitter   : %s" % job["submitter"])
        print("device-type : %s" % job["device_type"])
        print("device      : %s" % job["device"])
        print("health-check: %s" % job["health_check"])
        print("status      : %s" % job["status"])
        print("pipeline    : %s" % job["pipeline"])
        print("tags        : %s" % str(job["tags"]))
        print("visibility  : %s" % job["visibility"])
        print("submit time : %s" % job["submit_time"])
        print("start time  : %s" % job["start_time"])
        print("end time    : %s" % job["end_time"])


def handle_submit(proxy, options):
    job_id = proxy.scheduler.jobs.submit(options.definition.read())
    if isinstance(job_id, list):
        for job in job_id:
            print(job)
    else:
        print(job_id)


def handle(proxy, options, _):
    handlers = {
        "cancel": handle_cancel,
        "definition": handle_definition,
        "list": handle_list,
        "logs": handle_logs,
        "resubmit": handle_resubmit,
        "run": handle_run,
        "show": handle_show,
        "submit": handle_submit
    }
    return handlers[options.sub_sub_command](proxy, options)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# vim: set ts=4

# Copyright 2017 Rémi Duraffort
# This file is part of lavacli.
#
# lavacli is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# lavacli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with lavacli.  If not, see <http://www.gnu.org/licenses/>

from setuptools import setup

from lavacli import __version__

setup(
    name='lavacli',
    version=__version__,
    description='LAVA XML-RPC command line interface',
    author='Rémi Duraffort',
    author_email='ivoire@videolan.org',
    license="AGPLv3+",
    url='https://framagit.org/ivoire/lavacli',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Communications",
        "Topic :: Software Development :: Testing",
        "Topic :: System :: Networking",
    ],
    packages=['lavacli', 'lavacli.commands'],
    entry_points={
        'console_scripts': [
            'lavacli = lavacli:main'
        ]
    },
    install_requires=[
        "PyYAML",
        "pyzmq",
        "requests",
  ]
)
